<?php
namespace FiloBlu\ScratchAndWin\Controller\Adminhtml\ScratchAndWin;

use Magento\Backend\App\Action;

class Edit extends \Magento\Backend\App\Action
{
    /**
     * Core registry
     *
     * @var \Magento\Framework\Registry
     */
    protected $_coreRegistry = null;

    /**
     * @var \Magento\Framework\View\Result\PageFactory
     */
    protected $resultPageFactory;

    protected $scratchAndWin;

    protected $scratchAndWinTranslations;

    /**
     * @param Action\Context $context
     * @param \Magento\Framework\View\Result\PageFactory $resultPageFactory
     * @param \Magento\Framework\Registry $registry
     */
    public function __construct(
        Action\Context $context,
        \Magento\Framework\View\Result\PageFactory $resultPageFactory,
        \Magento\Framework\Registry $registry,
        \FiloBlu\ScratchAndWin\Model\ScratchAndWinFactory $scratchAndWin,
        \FiloBlu\ScratchAndWin\Model\ScratchAndWinTranslationsFactory $scratchAndWinTranslations
    ) {
        $this->resultPageFactory = $resultPageFactory;
        $this->_coreRegistry = $registry;
        $this->scratchAndWin = $scratchAndWin;
        $this->scratchAndWinTranslations = $scratchAndWinTranslations;
        parent::__construct($context);
    }

    /**
     * {@inheritdoc}
     */
    protected function _isAllowed()
    {
        return $this->_authorization->isAllowed('FiloBlu_ScratchAndWin::config');
    }

    /**
     * Init actions
     *
     * @return \Magento\Backend\Model\View\Result\Page
     */
    protected function _initAction()
    {
        // load layout, set active menu and breadcrumbs
        /** @var \Magento\Backend\Model\View\Result\Page $resultPage */
        $resultPage = $this->resultPageFactory->create();
        $resultPage->setActiveMenu('FiloBlu_ScratchAndWin::general_list')
            ->addBreadcrumb(__('ScratchAndWin'), __('ScratchAndWin'))
            ->addBreadcrumb(__('Manage ScratchAndWin'), __('Manage ScratchAndWin'));
        return $resultPage;
    }

    public function execute()
    {
        $id = $this->getRequest()->getParam('id');
        $storeId = $this->getRequest()->getParam('store',0);

        $model = $this->scratchAndWin->create();

        $modelTranslations = $this->scratchAndWinTranslations->create();

        if ($id) {
            $modelTranslations->loadByParentAndStoreId($id,$storeId);

            $model->load($id);
            if (!$model->getId()) {
                $this->messageManager->addError(__('This scratchandwin no longer exists.'));
                /** \Magento\Backend\Model\View\Result\Redirect $resultRedirect */
                $resultRedirect = $this->resultRedirectFactory->create();

                return $resultRedirect->setPath('*/*/');
            }
        }

        $this->_coreRegistry->register('filoblu_scratchandwin', $model);
        $this->_coreRegistry->register('filoblu_scratchandwin_translations', $modelTranslations);

        /** @var \Magento\Backend\Model\View\Result\Page $resultPage */
        $resultPage = $this->_initAction();
        $resultPage->addBreadcrumb(
            $id ? __('Edit ScratchAndWin') : __('New ScratchAndWin'),
            $id ? __('Edit ScratchAndWin') : __('New ScratchAndWin')
        );
        $resultPage->getConfig()->getTitle()->prepend(__('ScratchAndWin Page'));
        $resultPage->getConfig()->getTitle()
            ->prepend($model->getId() ? $model->getCode() : __('New ScratchAndWin'));

        return $resultPage;
    }
}