<?php
namespace FiloBlu\ScratchAndWin\Controller\Adminhtml\ScratchAndWinDetails;

use Magento\Backend\App\Action;

class Save extends \Magento\Backend\App\Action
{

    protected $uploaderFactory;
    protected $imageModel;
    protected $jsHelper;
    protected $scratchAndWinDetailsModelFactory;
    protected $scratchAndWinDetailsTranslationsModelFactory;

    /**
     * @param Action\Context $context
     */
    public function __construct(
        Action\Context $context,
        \Magento\MediaStorage\Model\File\UploaderFactory $uploaderFactory,
        \FiloBlu\ScratchAndWin\Model\ScratchAndWin\Image $imageModel,
        \Magento\Backend\Helper\Js $jsHelper,
        \FiloBlu\ScratchAndWin\Model\ScratchAndWinDetailsFactory $scratchAndWinDetailsModelFactory,
        \FiloBlu\ScratchAndWin\Model\ScratchAndWinDetailsTranslationsFactory $scratchAndWinDetailsTranslationsModelFactory
    )
    {
        $this->uploaderFactory = $uploaderFactory;
        $this->imageModel = $imageModel;
        $this->jsHelper = $jsHelper;
        $this->scratchAndWinDetailsModelFactory = $scratchAndWinDetailsModelFactory;
        $this->scratchAndWinDetailsTranslationsModelFactory = $scratchAndWinDetailsTranslationsModelFactory;
        parent::__construct($context);
    }

    /**
     * {@inheritdoc}
     */
    protected function _isAllowed()
    {
        return $this->_authorization->isAllowed('FiloBlu_ScratchAndWin::config');
    }

    /**
     * Save action
     *
     * @return \Magento\Framework\Controller\ResultInterface
     */
    public function execute()
    {
        $data = $this->getRequest()->getPostValue();
        /** @var \Magento\Backend\Model\View\Result\Redirect $resultRedirect */
        $resultRedirect = $this->resultRedirectFactory->create();

        if ($data) {

            $id = $this->getRequest()->getParam('id');

            $storeId = $this->getRequest()->getParam('store',0);
            $isSavingTranslation = false;

            if ($id && $storeId != 0) {
                $isSavingTranslation = true;
            }

            $model = $this->scratchAndWinDetailsModelFactory->create();

            $modelTranslations = $this->scratchAndWinDetailsTranslationsModelFactory->create();

            if ($id) {
                $model->load($id);

                if ($isSavingTranslation) {
                    $modelTranslations->loadByParentAndStoreId($id,$storeId);
                }
            }

            $model->setData($data);

            $bgImage = $this->uploadFileAndGetName('bg_image', $this->imageModel->getBaseDir(), $data);
            $model->setBgImage($bgImage);

            $overlayImage = $this->uploadFileAndGetName('overlay_image', $this->imageModel->getBaseDir(), $data);
            $model->setOverlayImage($overlayImage);
            
            $scratchedImage = $this->uploadFileAndGetName('scratched_image', $this->imageModel->getBaseDir(), $data);
            $model->setScratchedImage($scratchedImage);

            try {



                if ($isSavingTranslation) {

                    $modelTranslations->setSerializedValues(serialize($model->getData()));
                    $modelTranslations->setFilobluScratchandwinDetailsId($id);
                    $modelTranslations->setStoreId($storeId);
                    $modelTranslations->save();

                }
                else {
                    $this->_eventManager->dispatch(
                        'filoblu_scratchandwindetails_prepare_save',
                        [
                            'scratchandwindetails' => $model,
                            'request' => $this->getRequest()
                        ]
                    );

                    $model->save();
                }

                $this->messageManager->addSuccess(__('You saved this ScratchAndWin Details.'));
                if ($this->getRequest()->getParam('back')) {
                    return $resultRedirect->setPath('*/*/edit', ['id' => $model->getId() ? : $id, 'store' => $storeId, '_current' => true]);
                }
                return $resultRedirect->setPath('*/*/');
            } catch (\Magento\Framework\Exception\LocalizedException $e) {
                $this->messageManager->addError($e->getMessage());
            } catch (\RuntimeException $e) {
                $this->messageManager->addError($e->getMessage());
            } catch (\Exception $e) {
                $this->messageManager->addException($e, __('Something went wrong while saving the ScratchAndWin Details.'));
            }

            return $resultRedirect->setPath('*/*/edit', ['id' => $this->getRequest()->getParam('id'),'store' => $storeId]);
        }
        return $resultRedirect->setPath('*/*/');
    }

    public function uploadFileAndGetName($input, $destinationFolder, $data)
    {
        try {
            if (isset($data[$input]['delete'])) {
                return '';
            } else {
                $uploader = $this->uploaderFactory->create(['fileId' => $input]);
                $uploader->setAllowRenameFiles(true);
                $uploader->setFilesDispersion(true);
                $uploader->setAllowCreateFolders(true);
                $result = $uploader->save($destinationFolder);
                return $result['file'];
            }
        } catch (\Exception $e) {
            if ($e->getCode() != \Magento\Framework\File\Uploader::TMP_NAME_EMPTY) {
                throw new FrameworkException($e->getMessage());
            } else {
                if (isset($data[$input]['value'])) {
                    return $data[$input]['value'];
                }
            }
        }
        return '';
    }
}
